/**
 * Abstraction service to use Inapp purchases
 * @author Lukas Schönbächler
 * @see https://github.com/AlexDisler/cordova-plugin-inapppurchase
 */
angular.module('app.services')
    .factory('Store', function ($ionicPlatform, $log, Analytics, Utils, PACKAGES, FEATURES, CONSUMABLE, PackageDB, ItemDB, Profile, ITEMS, STORE_PREFIX) {

        // Calculate prefix for iOS (due to the fact that have global unique package names)
        var CALCULATED_STORE_PREFIX = '';
        if(ionic.Platform.isIOS()){
            CALCULATED_STORE_PREFIX = STORE_PREFIX;
        }

        var storeReady = false;
        var readyListener;

        /**
         * Marks the passed package as purchased
         * @param {pack}  pack  Entire package
         */
        var activatePackage = function (pack, showMessage) {

            // Mark package as purchased
            pack.purchased = true;
            PackageDB.update(pack);

            // Mark items in package as purchased
            var keys = Object.keys(ITEMS[pack.category][pack.id]);
            for (var i = 0; i < keys.length; i++) {
                // Get load item (remove ".csv" extension)
                var item = ItemDB.selectById(keys[i].slice(0, -4));
                item.purchased = true;
                ItemDB.insertOrUpdate(item);
            }

            // Set has purchases flag (no more ads for this user)
            Profile.setHasPurchases();

            // Show purchase successfull message to the user
            if(showMessage) Utils.showTranslatedMessage("PAKET_PURCHASED");
        };

        // Slices into smaller chunks because of https://github.com/AlexDisler/cordova-plugin-inapppurchase/blob/master/src/android/billing/IInAppBillingService.aidl#L61
        var recursivePurchase = function(remainingProductIds){
            inAppPurchase
                .getProducts(remainingProductIds.splice(0, 18))
                .then(function (products) {
                  $log.debug('Store: restored products, remaining: ' + remainingProductIds.length);
                    if (remainingProductIds.length == 0) {
                        storeReady = true;
                        $log.debug('Store: successfully initialized');
                        if(readyListener) readyListener();
                    } else {
                        $log.debug('Store: continue initialization');
                        recursivePurchase(remainingProductIds);
                    }
                })
                .catch(function (error) {
                    Analytics.trackException(error, false);
                    $log.warn('Store: initialization failed');
                    $log.error(error);
                });
        };

        /**
         * Purchase something in in the Play / App store
         * @param {string}   key        Error description
         * @param {function} success    Callback in case of success
         * @param {function} error      Callback in case of error
         */
        var order = function (key, success, error) {
            // Only available on physical devices
            if (ionic.Platform.isAndroid() || ionic.Platform.isIOS()) {

                // Make sure store is ready
                if (!storeReady && error) error("Store is not ready");
                inAppPurchase
                    .buy(key)
                    .then(function (data) {
                        if (success) success(data);
                    })
                    .catch(function (err) {
                        if (error) error(err);
                    });
            } else {
                if (success){
                  window.setTimeout(success, 500);
                }
            }
        };

        $ionicPlatform.ready(function () {

            // Load all unpurchased packages
            var productIds = [];
            var packages = PackageDB.selectAllNotPurchased();

            for (var i = 0; i < packages.length; i++) {
                productIds.push(CALCULATED_STORE_PREFIX + packages[i].id);
            }

            // Load features
            for (var i = 0; i < FEATURES.length; i++) {
                if (!Profile.getHasFunction(FEATURES[i])) {
                    productIds.push(CALCULATED_STORE_PREFIX + FEATURES[i]);
                }
            }

            // Load consumable
            for (var i = 0; i < CONSUMABLE.length; i++) {
                productIds.push(CALCULATED_STORE_PREFIX + CONSUMABLE[i]);
            }

            if (typeof inAppPurchase !== 'undefined') {
                try {
                  $log.log('Store: start recursivePurchase');
                  recursivePurchase(productIds);
                  storeReady = true;
                  $log.log('Store: started recursivePurchase');
                } catch (error) {
                    $log.warn('Store: Not active');
                    $log.error('Store error', error);
                    Analytics.trackException(error, false);
                }
            }else{
                $log.debug('Store: Not active - dev mode');
            }
        });

        var Store = {

            /**
             * Triggered as soon as the store is initialized
             * @param {function} listener   Listener method to be called
             */
            addReadyListener: function( listener ) {
                readyListener = listener;
                if(storeReady){
                    readyListener();
                }
            },

            /**
             * Purchase a package
             * @param {pack}     pack       Entire package
             * @param {function} success    Callback in case of success
             * @param {function} error      Callback in case of error
             */
            purchasePackage: function (pack, success, error) {
                order(CALCULATED_STORE_PREFIX + pack.id, function () {
                    activatePackage(pack);
                    $log.debug('Store: Pack ' + pack.id + " successfull purchased");
                    if (success) success();
                }, function (error) {
                    if (ionic.Platform.isAndroid()) {
                        if (error.response == 7) {
                            // Already owned
                            activatePackage(pack);
                            $log.warn('Store: Pack ' + pack.id + " was already purchased");
                            if (success) success();
                        } else if (error.response == -1003) {
                            // Verification failed (testing case)
                            activatePackage(pack);
                            $log.warn('Store: Pack ' + pack.id + " not verified but purchased (should just happen in debug build)");
                            if (success) success();
                        } else {
                            Analytics.trackException(error, false);
                            $log.error('Store: Pack ' + pack.id + " purchase failed");
                            if (angular.isFunction(error)) error();
                        }
                    } else {
                        Analytics.trackException(error, false);
                        $log.warn('Store: Pack ' + pack.id + " purchase failed");
                        $log.error(error);
                        if (error) error();
                    }
                });
            },

            /**
             * Purchase 10 hints
             * @param {function} success    Callback in case of success
             * @param {function} error      Callback in case of error
             */
            purchaseHints: function (success, error) {

                var hints = CALCULATED_STORE_PREFIX + 'hints_10';
                if(CALCULATED_STORE_PREFIX == 'bin_'){
                    hints += "_fix";
                }

                // Start purchase call
                order(hints, function (data) {

                    // Directly consume hints after success, because in Android you cannot purchase consumable goods if you haven't consumed them and we don't want to keep the receipt
                    if (ionic.Platform.isAndroid()) {
                        var worked = inAppPurchase.consume(data.productType, data.receipt, data.signature).then(function () {
                            $log.debug('Store: Hint successfully consumed');
                        }, function (error) {
                            $log.error('Store: Consuming hints failed');
                            Analytics.trackException(error, false);
                        });
                    }

                    // Increase hints counter in profile
                    Profile.addHints();

                    // Provide some user feedback
                    Utils.showTranslatedMessage("HINTS_PURCHASED");

                    if (success) success();
                    $log.debug('Store: Purchasing hints successfull');
                }, function (error) {
                    Utils.showMessage(e.message);
                    if (error) error();
                    Analytics.trackException(error, false);
                    $log.warn('Store: Purchasing hints failed');
                    $log.error(error);
                });
            },

            purchaseFunction: function (key, success, error) {
                var originalKey = key;
                key = CALCULATED_STORE_PREFIX + key;
                order(key, function () {
                    Profile.setHasFunction(originalKey);
                    if (success) success();
                }, function (e) {
                    if (ionic.Platform.isAndroid()) {
                        if (e.response == 7) {
                            // Already owned
                            Profile.setHasFunction(originalKey);
                            $log.warn('Store: Feature ' + key + " was already purchased");
                            if (success) success();
                        } else {
                            Analytics.trackException(error, false);
                            $log.error('Store: Purchasing ' + key + ' failed');
                            if (error) error();
                        }
                    } else {
                        Analytics.trackException(error, false);
                        $log.warn('Store: Purchasing ' + key + ' failed');
                        $log.error(error);
                        if (error) error();
                    }
                });
            },

            restorePurchases: function () {
                inAppPurchase
                    .restorePurchases()
                    .then(function (data) {
                        if (data && data.length > 0) {
                            for (var i = 0; i < data.length; i++) {
                                if ((data[i].productId == CALCULATED_STORE_PREFIX + "hints_10" || data[i].productId == "bin_hints_10_fix") &&  ionic.Platform.isAndroid()) {
                                    inAppPurchase.consume(data[i].productType, data[i].receipt, data[i].signature);
                                }

                                if (data[i].productId != CALCULATED_STORE_PREFIX + "hints_10" && data[i].productId != "bin_hints_10_fix" && ionic.Platform.isIOS()) {
                                    activatePackage(PackageDB.selectById(data[i].productId.substr(CALCULATED_STORE_PREFIX.length, data[i].productId.length)), false);
                                }

                                if (data[i].productId != CALCULATED_STORE_PREFIX + "hints_10" && data[i].productId != "bin_hints_10_fix" && ionic.Platform.isAndroid() && JSON.parse(data[i].receipt).purchaseState == 0) {
                                    activatePackage(PackageDB.selectById(data[i].productId.substr(CALCULATED_STORE_PREFIX.length, data[i].productId.length)), false);
                                }
                            }
                        }

                        Utils.showTranslatedMessage('PURCHASES_RESTORED');
                        $log.debug("Store: Restore successfull, " + data.length + " items restored");

                    })
                    .catch(function (error) {
                        Analytics.trackException(error, false);
                        $log.warn('Store: Restoring failed');
                        $log.error(error);
                        //Utils.showMessage(error.message);
                    });
            }
        };
        return Store;
    });
